@@ -152,6 +152,7 @@ enum ofono_atom_type {
 	OFONO_ATOM_TYPE_CDMA_NETREG,
 	OFONO_ATOM_TYPE_HANDSFREE,
 	OFONO_ATOM_TYPE_SIRI,
+	OFONO_ATOM_TYPE_NETMON,
 };
 
 enum ofono_atom_watch_condition {
@@ -529,3 +530,5 @@ ofono_bool_t __ofono_cdma_provision_get_name(const char *sid, char **name);
 void __ofono_private_network_release(int id);
 ofono_bool_t __ofono_private_network_request(ofono_private_network_cb_t cb,
 						int *id, void *data);
+
+#include <ofono/netmon.h>
